from __future__ import print_function
# Libraries used

import cmath

from matplotlib import cm
import matplotlib.image as mpimg
import matplotlib.lines as mlines
import matplotlib.pyplot as plt
import matplotlib as mpl

from mpl_toolkits.mplot3d import Axes3D

import numpy as np
import os
import pandas as pd
import re
import seaborn as sns



from ipywidgets import interact, interactive, fixed, interact_manual
import ipywidgets as widgets


# Reference Values

refL = 50e-9 # m/s; log reference for structure-borne noise
refL2 = np.square(refL)


# One-third octave band calculations
size = 7

f_thirdlow = np.empty((size))
f_thirdlow[:] = np.nan

f_oct = np.empty((size))
f_oct[:] = np.nan

f_thirdhigh = np.empty((size))
f_thirdhigh[:] = np.nan

f_third = np.empty((30))
f_third[:] = np.nan


f_sixth = np.empty((60))
f_sixth[:] = np.nan


f = 1000
for i in range(6,-1,-1):
    flow = f/2**(1/3)
    fhigh = f*2**(1/3)
    f_oct[i] = f
    f_thirdlow[i] = flow
    f_thirdhigh[i] = fhigh
    f = f/2

f = 4000
for i in range(28,-1,-3):
    flow = f/2**(1/3)
    fhigh = f*2**(1/3)
    f_third[i] = f
    f_third[i-1] = flow
    f_third[i+1] = fhigh
    f = f/2
    
f = 4000
for i in range(59,-1,-6):
    flow3rd = f/2**(1/3)
    fhigh3rd = f*2**(1/3)
    flow3rd_low6th = flow3rd/2**(1/6)
    flow3rd_high6th = flow3rd*2**(1/6)
    fhigh3rd_low6th = fhigh3rd/2**(1/6)
    fhigh3rd_high6th = fhigh3rd*2**(1/6)
    
    f_sixth[i] = fhigh3rd_high6th 
    f_sixth[i-1] = fhigh3rd 
    f_sixth[i-2] = fhigh3rd_low6th 
    f_sixth[i-3] = f 
    f_sixth[i-4] = flow3rd_high6th
    f_sixth[i-5] = flow3rd 
    
    f = f/2

    f_3rd = np.round(f_third)
    f_3rd = f_3rd.astype(int)
    
    f_6th = np.round(f_sixth)
    f_6th = f_6th.astype(int)


# A Weighting: 50Hz to 3150 Hz bands
Aweight = np.empty((19,0))
Aweight = [-30.2, -26.2, -22.5, -19.1, -16.1, -13.4, -10.9, -8.6, -6.6, -4.8, -3.2, -1.9, -0.8, 0, 0.6, 1.0, 1.2, 1.3, 1.2] # 50, 63, 80, 100, 125, 160, 200, 250, 315, 400, 500, 630, 800, 1000, 1250, 1600, 2000, 2500, 3150 [Hz]
Aweight = np.asarray(Aweight)



# Function to read in data from the Apollo acquisition system

def ApolloSystemBode1(file,lines2skip):

    ## Read in data
    data = pd.read_csv(file,sep=';',header=lines2skip);
    data = pd.read_csv(file,sep=';',header=lines2skip);
    data = data.to_numpy() #convert from pandas dataframe to numpy array
    data = data[:,1:-1] #last column is NaNs so remove
    
    ## Convert to format that python can read (note pandas reads on 'j' as complex)
    i=0
    nCols = data.size
    data_phs = np.empty(nCols)
    data = data[0,0:nCols]
    for s in data:
        s = s.replace('i', 'j')
        data[i] = complex(s)
        data_phs[i] = np.angle(data[i],deg=True)
        i=i+1

    data_abs = abs(data)
    data_abs = data_abs.astype(np.float)
    
    return data_abs, data_phs



# Function to read in data from the Apollo acquisition system: 2 is for when over and under range boxes are ticked

def ApolloSystemBode2(file,lines2skip):

    ## Read in data
    data = pd.read_csv(file,sep=';',header=lines2skip);
    data = pd.read_csv(file,sep=';',header=lines2skip);
    data = data.to_numpy() #convert from pandas dataframe to numpy array
    data = data[:,3:-1] #last column is NaNs so remove
    
    ## Convert to format that python can read (note pandas reads on 'j' as complex)
    i=0
    nCols = data.size
    data_phs = np.empty(nCols)
    data = data[0,0:nCols]
    for s in data:
        s = s.replace('i', 'j')
        data[i] = complex(s)
        data_phs[i] = np.angle(data[i],deg=True)
        i=i+1

    data_abs = abs(data)
    data_abs = data_abs.astype(np.float)
    
    return data_abs, data_phs



# Function to read in force data from the Apollo acquisition system

def ApolloSystemChFrce(file,lines2skip):
    
    data_F= pd.read_csv(file,sep=';',header=lines2skip);
    data_F = data_F.to_numpy() #convert from pandas dataframe to numpy array
    data_F = data_F[:,1:-3] #last column is NaNs so remove
    data_F = data_F.astype(np.float)
    nCols = data_F.size
    data_F = data_F[0,0:nCols]
    
    return data_F


# Function to read in data from text files extracted from the PSV .svd files

def PSVbode(filef,fileimag,filereal):
    
    # Read in the frequency spectrum
    f = pd.read_csv(filef,header=0); 
    f = f.to_numpy() #convert from pandas dataframe to numpy array
    
    # Read in the Imaginary part of the FRF
    FRFImag = pd.read_csv(fileimag,header=None,delim_whitespace=True);
    FRFImag = FRFImag.transpose()
    FRFImag = FRFImag.to_numpy() #convert from pandas dataframe to numpy array
    
    # Read in the Imaginary part of the FRF
    FRFReal = pd.read_csv(filereal,header=None,delim_whitespace=True);
    FRFReal = FRFReal.transpose()
    FRFReal = FRFReal.to_numpy() #convert from pandas dataframe to numpy array
    
    return f, FRFImag, FRFReal


# Function to calculate square of RMS

def sRMS(frfImag, frfReal):
    
    Row = 0
    Col = 1

    Vms = np.square(frfImag) + np.square(frfReal);
    VmsSptAvg = np.mean(Vms,axis=Col)
    
    return Vms, VmsSptAvg



# Function to read in Coherence data and calculate the signal to noise ratio

def cohRead(file):
    coh = pd.read_csv(file, header=None, delim_whitespace=True);
    coh = coh.transpose()
    coh = coh.to_numpy() #convert from pandas dataframe to numpy array
    
    coh = np.around(coh,5)
    dcoh = 1-coh
    snr = np.divide(coh,dcoh,out=np.zeros_like(coh),where=dcoh!=0)
    indices = np.where(snr == 0)
    snr[indices[0],indices[1]] = np.max(snr)
    
    return coh, snr


# Function to read in the forcing functions and calculate their mean, standard deviation, maxima and minima as a function of frequency 

def FRead(file):
    
    F = pd.read_csv(file, header=None, delim_whitespace=True);
    F = F.transpose()
    F = F.to_numpy() #convert from pandas dataframe to numpy array

    Col = 1

    Fstd = np.std(F,Col)
    Fmean = np.mean(F,Col)
    Fmax = np.amax(F,Col)
    Fmin = np.amin(F,Col)
    
    return F, Fstd, Fmean, Fmax, Fmin


# Make a bodeplot (not used, just left as template)

def bodeplot(mag,phs):

    SMALLEST_SIZE = 12
    SMALL_SIZE = 15
    MEDIUM_SIZE = 20
    BIGGER_SIZE = 25

    plt.rc('font', size=SMALL_SIZE)          # controls default text sizes
    # plt.rc('axes', titlesize=MEDIUM_SIZE)     # fontsize of the axes title
    plt.rc('axes', labelsize=MEDIUM_SIZE)    # fontsize of the x and y labels
    plt.rc('xtick', labelsize=SMALLEST_SIZE)    # fontsize of the tick labels
    plt.rc('ytick', labelsize=SMALLEST_SIZE)    # fontsize of the tick labels
    plt.rc('legend', fontsize=SMALLEST_SIZE)    # legend fontsize
    plt.rc('figure', titlesize=BIGGER_SIZE)  # fontsize of the figure title
    
    fig, (ax1, ax2) = plt.subplots(2)
    fig.suptitle('Measurement 3 (Ch3vCh1) – vs – Measurment 4 (Ch3vCh1)')
    
    ax1.set_facecolor('#FAFAFA')
    ax1.semilogx(f[f_LowIndex:f_UpIndex], 10*np.log10(M3_CH3vCH1_abs[f_LowIndex:f_UpIndex]), label='Measurement 3 (Ch3vCh1)')
    ax1.semilogx(f[f_LowIndex:f_UpIndex], 10*np.log10(M4_CH3vCH1_abs[f_LowIndex:f_UpIndex]), label='Measurment 4 (Ch3vCh1)')
    ax1.legend(loc="lower right")
    ax1.set(xlabel='Frequency (Hz)', ylabel='Mechanical Mobility (dB ref. 1 $\\frac{ms^{-1}}{N}$)')
    ax1.grid(True, color='k', linestyle='--', linewidth=1, alpha=0.2)
    #ax1.set_xticks([8,16,31.5,63,125,250,500,1000,2000,4000])
    #ax1.set_xticks([8,10,12.5,16,20,25,31.5,40,50,63,80,100,125,160,200,250,315,400,500,630,800,1000,1250,1600,2000,2500,3150,4000])
    ax1.set_xticks(f_6th)
    ax1.get_xaxis().set_major_formatter(mpl.ticker.ScalarFormatter())
    ax1.set_xlim([f_LowLim, f_UpLim])
    ax1.set_ylim([-35, 5])
    
    ax2.set_facecolor('#FAFAFA')
    ax2.semilogx(f[f_LowIndex:f_UpIndex], M3_CH3vCH1_phs[f_LowIndex:f_UpIndex], label='Measurement 3 (Ch3vCh1)')
    ax2.semilogx(f[f_LowIndex:f_UpIndex], M4_CH3vCH1_phs[f_LowIndex:f_UpIndex], label='Measurment 4 (Ch3vCh1)')
    ax2.legend(loc="lower right")
    ax2.set(xlabel='Frequency (Hz)', ylabel='Phase (deg)')
    ax2.grid(True, color='k', linestyle='--', linewidth=1, alpha=0.2)
    ax2.set_xticks(f_6th)
    ax2.get_xaxis().set_major_formatter(mpl.ticker.ScalarFormatter())
    ax2.set_xlim([f_LowLim, f_UpLim])
    
    
    #for i in range(0,58,2):
        #ax2.get_xticklabels()[10].set_color('red')
        #ax2.get_xticklabels()[8].set_fontsize(20)
        #ax2.get_xticklabels()[i].set_weight("bold")
        #ax2.get_xticklabels()[i].set_rotation(45)
        #ax2.get_xticklabels()[i+1].set_rotation(45)
    
    ax2.set_ylim([-180, 180])
    
    return 0


# Function to read in .csv files

def ANSYScsvRead(fileimag,filereal):

    FRFImag = pd.read_csv(fileimag,sep=',',header=None);
    FRFImag = FRFImag.to_numpy() #convert from pandas datafram to numpy array
    FRFImag = FRFImag[:,0:-1] #last column is NaNs so remove
    FRFImag = FRFImag.transpose()

    
    FRFReal = pd.read_csv(filereal,sep=',',header=None);
    FRFReal = FRFReal.to_numpy() #convert from pandas datafram to numpy array
    FRFReal = FRFReal[:,0:-1] #last column is NaNs so remove
    FRFReal = FRFReal.transpose()

    
    return FRFImag, FRFReal


# Function to read in .csv files

def ANSYScsvRead2(fileline,fileimag,filereal):

    FRFFreq = pd.read_csv(fileline,sep=',',header=None);
    FRFFreq = FRFFreq.to_numpy() #convert from pandas datafram to numpy array
    
    FRFImag = pd.read_csv(fileimag,sep=',',header=None);
    FRFImag = FRFImag.to_numpy() #convert from pandas datafram to numpy array
    FRFImag = FRFImag[:,0:-1] #last column is NaNs so remove
    FRFImag = FRFImag.transpose()
    
        
    FRFReal = pd.read_csv(filereal,sep=',',header=None);
    FRFReal = FRFReal.to_numpy() #convert from pandas datafram to numpy array
    FRFReal = FRFReal[:,0:-1] #last column is NaNs so remove
    FRFReal = FRFReal.transpose()

    
    return FRFFreq, FRFImag, FRFReal


# Function to read in the forcing functions and calculate their mean, standard deviation, maxima and minima as a function of frequency 

def FRead(file):
    
    F = pd.read_csv(file, header=None, delim_whitespace=True);
    F = F.transpose()
    F = F.to_numpy() #convert from pandas dataframe to numpy array

    Col = 1

    Fstd = np.std(F,Col)
    Fmean = np.mean(F,Col)
    Fmax = np.amax(F,Col)
    Fmin = np.amin(F,Col)
    
    return F, Fstd, Fmean, Fmax, Fmin


# Function to read in the calculated radiated Sound Power Levels (SWL) of the plate and calculate the equivalent Sound Pressure Levels (SPL) in the test facility.
# Note: Starts at the 50 Hz one-third octave band ! band50Hz = 9

def SWL2SPL(file,fThirdStart,fThirdEnd): 

    freqCol = 0
    SWLCol = 1
    c0 = 340 # (m.s^-1) Speed of sound in air
    rho0 = 1.2 # (kg.m^-3) density of Air
    A = 10
    refSPL = 2e-5 # reference Sound Pressure Level
    band50Hz = 9
    
    SWL = pd.read_csv(file,sep=',',header=0);
    SWL = SWL.to_numpy() #convert from pandas datafram to numpy array
    SWL = SWL.astype(np.float)
    SWL[:,freqCol] = np.round(SWL[:,freqCol],1) #Round to 1 d.p. to get rid of numerical errors

    f_LowIndex = int(np.where(SWL[:,freqCol]  == fThirdStart)[0])
    f_UpIndex = int(np.where(SWL[:,freqCol] == fThirdEnd)[0]) + 1 # add plus 1 due to numpy indexing
    
    SPL2 = 2*c0*rho0*SWL[f_LowIndex:f_UpIndex,SWLCol]/(A) # [Pa^2] squared sound pressure level
    SPL = np.sqrt(SPL2)
    Ln = 10*np.log10(SPL2/refSPL**2)
    freq = SWL[f_LowIndex:f_UpIndex,freqCol] 

    f = SWL[:,freqCol] 
    
    return freq, Ln


### Read in the the frequency steps used in the harmonic analysis

def ANSYScsvReadFreqs(file):

    freq = pd.read_csv(file,sep=',',header=None);
    
    return freq


# Function to read in the calculated radiated sound power of the plate and convert it to Sound Power Levels (SWL).

def readSWL2(file,fThirdStart,fThirdEnd): 

    freqCol = 0
    SWLCol = 1
    c0 = 340 # (m.s^-1) Speed of sound in air
    rho0 = 1.2 # (kg.m^-3) density of Air
    A = 10
    refSWL = 1e-12 # reference Sound Power Level: 1pW

    
    SWL = pd.read_csv(file,sep=',',header=0);
    SWL = SWL.to_numpy() #convert from pandas datafram to numpy array
    SWL = SWL.astype(np.float)
    SWL[:,freqCol] = np.round(SWL[:,freqCol],1) #Round to 1 d.p. to get rid of numerical errors

    f_LowIndex = int(np.where(SWL[:,freqCol]  == fThirdStart)[0])
    f_UpIndex = int(np.where(SWL[:,freqCol] == fThirdEnd)[0]) + 1 # add plus 1 due to numpy indexing
    
    freq = SWL[f_LowIndex:f_UpIndex,freqCol] 
    Lw = 10*np.log10(SWL[f_LowIndex:f_UpIndex,SWLCol]/(refSWL)) # SWL

   
    return freq, Lw


# Function to read in the ESL material properties calculated with the ESAComp software - this function is specific for the Flexural ESL Material Properties

def readESACompMPs(fileModuli,fileRatios): 
    
    Row = 0
    Col = 1

    ESAComp_Moduli = np.genfromtxt(fileModuli,delimiter=';',skip_header=2);
    ESAComp_Ratios = np.genfromtxt(fileRatios,delimiter=';',skip_header=2);   
    
    EC_ESAComp_f_temp = np.vstack((ESAComp_Moduli[90:,0], ESAComp_Moduli[90:,3], ESAComp_Moduli[0:91,3], ESAComp_Ratios[90:,2], ESAComp_Moduli[90:,4], ESAComp_Moduli[0:91,5], \
                                   ESAComp_Moduli[90:,5]));
    EC_ESAComp_f_temp = np.transpose(EC_ESAComp_f_temp)

    EC_ESAComP_f_temp_flip = np.flip(EC_ESAComp_f_temp,axis=Row);

    EC_ESAComp_f = np.vstack((EC_ESAComp_f_temp[0:-1,:],EC_ESAComP_f_temp_flip[0:-1,:],EC_ESAComp_f_temp[0:-1,:], EC_ESAComP_f_temp_flip[0:,:]));
   
    return EC_ESAComp_f


# Calculates the Universal Elastic Anisotropic Index
# References:   Ranganathan, S. I., & Ostoja-Starzewski, M. (2008). Universal elastic anisotropy index. Physical Review Letters, 101(5), 055504.
#               Hill, R. (1952). The elastic behaviour of a crystalline aggregate. Proceedings of the Physical Society. Section A, 65(5), 349.

def getUEAI(fileModuli,fileRatios): 
    
    Row = 0;
    Col = 1;

    ## Data
    ESAComp_Moduli = np.genfromtxt(fileModuli,delimiter=';',skip_header=2);
    ESAComp_Ratios = np.genfromtxt(fileRatios,delimiter=';',skip_header=2); 
    
    EC_ESAComp_f_temp = np.vstack((ESAComp_Moduli[90:,0], ESAComp_Moduli[90:,3], ESAComp_Moduli[0:91,3], \
                               ESAComp_Ratios[90:,2], ESAComp_Moduli[90:,4], ESAComp_Moduli[0:91,5], \
                               ESAComp_Moduli[90:,5]));
    EC_ESAComp_f_temp = np.transpose(EC_ESAComp_f_temp)

    EC_ESAComP_f_temp_flip = np.flip(EC_ESAComp_f_temp,axis=Row);

    EC_ESAComp_f = np.vstack((EC_ESAComp_f_temp[0:-1,:],EC_ESAComP_f_temp_flip[0:-1,:],EC_ESAComp_f_temp[0:-1,:], EC_ESAComP_f_temp_flip[0:,:]));


    ## Calc

    s = np.zeros((6,6));

    E1 = EC_ESAComp_f[0,1]
    E2 = EC_ESAComp_f[0,2]
    E3 = 0.999

    nu12 = EC_ESAComp_f[0,3]
    nu23 = 0.3
    nu13 = nu12

    G12 = EC_ESAComp_f[0,4]
    G23 = EC_ESAComp_f[0,5]
    G13 = EC_ESAComp_f[0,6]


    s[0,0] = 1/E1;
    s[1,1] = 1/E2;
    s[2,2] = 1/E3;
    s[3,3] = 1/G23;
    s[4,4] = 1/G13;
    s[5,5] = 1/G12;

    s[1,0] = nu12/E1;
    s[2,0] = nu13/E1;
    s[2,1] = nu23/E2;

    s[0,1] = s[1,0];
    s[0,2] = s[2,0];
    s[1,2] = s[2,1];

    C = np.linalg.inv(s);

    Kv = (C[0,0]+C[1,1]+C[2,2] + 2*(C[0,1]+C[1,2]+C[2,0]))/9;

    Gv = (C[0,0]+C[1,1]+C[2,2] - (C[0,1]+C[1,2]+C[2,0]) + 3*(C[3,3]+C[4,4]+C[5,5]))/15

    Kr = 1/(s[0,0]+s[1,1]+s[2,2]+2*(s[0,1]+s[1,2]+s[2,0]))

    Gr = 15/(4*(s[0,0]+s[1,1]+s[2,2])-4*(s[0,1]+s[1,2]+s[2,0])+3*(s[3,3]+s[4,4]+s[5,5]))

    Au = 5*(Gv/Gr) + Kv/Kr - 6
    
    return Au


def ABH0(r): 
    
    h0 = 0.2
    z0 = 0.04
    rC = 1
    m=4
    h = ((h0-z0)/(rC**m))*r**4 + z0
    
    return h


def ABH0INV(h): 
    
    h0 = 0.2
    z0 = 0.04
    rC = 1
    m=4
    r = (((rC**m)/(h0-z0))*(h-z0))**(1/m)
    
    return r


# Plots mode shapes extracted from Ansys with APDL modeExport_vXPX.mac

def modeplot(fileFreqs, fileCoords, fileDisp, fileMetadata, modeNum):
    
    modalFreqs = np.genfromtxt(fileFreqs, delimiter=',') # import the eigenfrequencies
    
    modalCoords = np.genfromtxt(fileCoords, delimiter=',') # import the modal coordinates
    modalCoords = modalCoords[:,0:-1]
    
    modalDisp = np.genfromtxt(fileDisp, delimiter=',') # import the modal displacements
    modalDisp = modalDisp[:,0:-1]
    
    modalMetadata = np.genfromtxt(fileMetadata, delimiter=',') # import the modal metadata
    modalMetadata = modalMetadata.astype(int)
    nCols = modalMetadata[1]
    nRows = modalMetadata[0]
    
    # Sort the data 
    xCoords = 0
    yCoords = 1
    
    ind_xv = np.lexsort((modalCoords[:,xCoords],modalCoords[:,yCoords])) # get the x-coordinate vector
    xv = modalCoords[ind_xv]
    xv = xv[0:nCols,xCoords]
    
    ind_yv = np.lexsort((modalCoords[:,yCoords],modalCoords[:,xCoords])) # get the y-coordinate vector
    yv = modalCoords[ind_yv]
    yv = yv[0:nRows,yCoords]
    
    # Generate Mesh
    xMesh, yMesh = np.meshgrid(xv,-yv)
    Z = np.reshape(modalDisp[ind_xv,modeNum],(nRows,nCols))



    return xMesh, yMesh, Z


# Plots the Average Driving Point Residue (ADPR) with modes extracted from modeExport_vXPX.mac
# Lower the value, the better the location for supports
# Ref: Ewins, D. J. (2000). Modal testing: theory, practice and application. 2ed. John Wiley & Sons. Section 6.6.2.2.


def ADPR(fileFreqs, fileCoords, fileDisp, fileMetadata, nModes):
    
    modalFreqs = np.genfromtxt(fileFreqs, delimiter=',') # import the eigenfrequencies
    
    modalCoords = np.genfromtxt(fileCoords, delimiter=',') # import the modal coordinates
    modalCoords = modalCoords[:,0:-1]
    
    modalDisp = np.genfromtxt(fileDisp, delimiter=',') # import the modal displacements
    modalDisp = modalDisp[:,0:-1]
    
    modalMetadata = np.genfromtxt(fileMetadata, delimiter=',') # import the modal metadata
    modalMetadata = modalMetadata.astype(int)
    nCols = modalMetadata[1]
    nRows = modalMetadata[0]
    
    # Sort the data 
    xCoords = 0
    yCoords = 1
    
    ind_xv = np.lexsort((modalCoords[:,xCoords],modalCoords[:,yCoords])) # get the x-coordinate vector
    xv = modalCoords[ind_xv]
    xv = xv[0:nCols,xCoords]
    
    ind_yv = np.lexsort((modalCoords[:,yCoords],modalCoords[:,xCoords])) # get the y-coordinate vector
    yv = modalCoords[ind_yv]
    yv = yv[0:nRows,yCoords]
    
    
    # Calculate the ADPR
    modalDisp2 = np.square(modalDisp)
    modalFreqs2 = np.square(modalFreqs)
    ADPR = np.divide(modalDisp2,modalFreqs2)
    ADPR = np.sum(ADPR[:,0:nModes],1)

    
    # Generate Mesh
    xMesh, yMesh = np.meshgrid(xv,-yv)
    Z = np.reshape(ADPR[ind_xv],(nRows,nCols))
    



    return xMesh, yMesh, Z


# Plots Optimium Driving Point (ODP) with modes extracted from Ansys with APDL modeExport_vXPX.mac
# Larger the value, the better the location for exciting all of the modes
# Ref: Ewins, D. J. (2000). Modal testing: theory, practice and application. 2ed. John Wiley & Sons. Section 6.6.2.2.

def ODP(fileFreqs, fileCoords, fileDisp, fileMetadata, modeStart, modeEnd, modesToDelete):
    
    modalFreqs = np.genfromtxt(fileFreqs, delimiter=',') # import the eigenfrequencies
    
    modalCoords = np.genfromtxt(fileCoords, delimiter=',') # import the modal coordinates
    modalCoords = modalCoords[:,0:-1]
    
    modalDisp = np.genfromtxt(fileDisp, delimiter=',') # import the modal displacements
    modalDisp = modalDisp[:,0:-1]
    
    modalMetadata = np.genfromtxt(fileMetadata, delimiter=',') # import the modal metadata
    modalMetadata = modalMetadata.astype(int)
    nCols = modalMetadata[1]
    nRows = modalMetadata[0]
    
    # Sort the data 
    xCoords = 0
    yCoords = 1
    
    ind_xv = np.lexsort((modalCoords[:,xCoords],modalCoords[:,yCoords])) # get the x-coordinate vector
    xv = modalCoords[ind_xv]
    xv = xv[0:nCols,xCoords]
    
    ind_yv = np.lexsort((modalCoords[:,yCoords],modalCoords[:,xCoords])) # get the y-coordinate vector
    yv = modalCoords[ind_yv]
    yv = yv[0:nRows,yCoords]
    
    
    # Calculate the ODP
    modalDisp_clean = np.delete(modalDisp,modesToDelete,1) # remove unwanted modes
    ODP = np.prod(np.abs(modalDisp_clean[:,modeStart:modeEnd]),1)

    
    # Generate Mesh
    xMesh, yMesh = np.meshgrid(xv,-yv)
    Z = np.reshape(ODP[ind_xv],(nRows,nCols))
    



    return xMesh, yMesh, Z
